@@ -4,7 +4,7 @@ from typing import Union, List, Dict
 import aiohttp
 import requests
 
-from quantel.exceptions import InvalidAPIKey, GatewayError
+from quantel.exceptions import InvalidAPIKey, GatewayError, TooManyRequests
 
 
 class _Ticker:
@@ -19,7 +19,17 @@ class _Ticker:
 
         self.symbols = list(self._chunks(symbols, 30))
 
-    def _get_data(self, endpoint, **kwargs) -> List[Dict]:
+    def _get_data(self, endpoint: str, **kwargs) -> List[Dict]:
+        """
+
+
+        Args:
+            endpoint: Quantel API endpoint
+        Kwargs:
+            months: Filter results by last n months (insider_transactions & insider_transactions_summarized)
+
+
+        """
 
         headers = {
             "x-rapidapi-key": self.api_key,
@@ -35,7 +45,7 @@ class _Ticker:
         flat = [x for sublist in result for x in sublist]
         return flat
 
-    async def _submit_async(self, headers, endpoint, **kwargs):
+    async def _submit_async(self, headers: dict, endpoint: str, **kwargs):
         async with aiohttp.ClientSession(headers=headers) as session:
             tasks = []
             for chunk in self.symbols:
@@ -44,12 +54,13 @@ class _Ticker:
             res = await asyncio.gather(*tasks)
             return res
 
-    async def _get_data_async(self, session, endpoint, symbols, **kwargs):
+    async def _get_data_async(self, session: aiohttp.ClientSession, endpoint: str, symbols: str, **kwargs) -> List[
+        Dict]:
         async with session.get(f"{self.host}{endpoint}/{symbols}", params=kwargs) as response:
-            if response.status == 200:
+            if self._check_status(response.status):
                 return await response.json()
 
-    def _submit_sync(self, headers, endpoint, **kwargs):
+    def _submit_sync(self, headers: dict, endpoint: str, **kwargs):
         tasks = []
 
         session = requests.Session()
@@ -60,18 +71,33 @@ class _Ticker:
 
         return tasks
 
-    def _get_data_sync(self, session, endpoint, symbols, **kwargs):
+    def _get_data_sync(self, session: requests.Session, endpoint: str, symbols: str, **kwargs) -> List[Dict]:
 
         res = session.get(f"{self.host}{endpoint}/{symbols}", params=kwargs)
 
-        if res.status_code == 200:
+        if self._check_status(res.status_code):
+            print(res.json())
             return res.json()
 
+    def _check_status(self, status_code: int) -> bool:
+        """
+        Check the status code from response
+
+        Args:
+            status_code: HTTP response code
+        """
+
+        if status_code == 429:
+            raise TooManyRequests(
+                "You may have exceeded the MONTHLY quota for your current plan. Upgrade your plan at http://links.quantel.io/upgrade")
+
+        return True
+
     def _chunks(self, l, n):
         n = max(1, n)
         return (l[i:i + n] for i in range(0, len(l), n))
 
-    def income_statement(self) -> List[Dict]:
+    def income_statement(self, period: str = None) -> List[Dict]:
         """
         Get income statements
 
@@ -94,10 +120,34 @@ class _Ticker:
 
         - N/A
 
-        Returns:
+        """
+        return self._get_data("income-statement", period=period)
+
+    def income_statement_ttm(self) -> List[Dict]:
+        """
+        Get income statement trailing twelve months
+
+        Exchanges supported
+
+        - NASDAQ
+        - NYSE (New York Stock Exchange)
+        - XETRA (German Electronic Exchange)
+        - ASX (Australian Stock Exchange)
+        - TSX (Toronto Stock Exchange)
+        - ENX (EuroNext)
+        - NSE (National Stock Exchange of India)
+        - LSE (London Stock Exchange)
+        - MOEX (Moscow Stock Exchange)
+        - HKEX (Hong Kong Stock Exchange)
+        - SIX (Swiss Stock Exchange)
+        - OSE (Oslo Stock Exchange)
+
+        Not Supported.
+
+        - N/A
 
         """
-        return self._get_data("income-statement")
+        return self._get_data("income-statement-ttm")
 
     def income_statement_growth(self) -> List[Dict]:
         """
@@ -122,11 +172,10 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("income-statement-growth")
 
-    def balance_sheet(self) -> List[Dict]:
+    def balance_sheet(self, period: str = None) -> List[Dict]:
         """
         Get balance sheets
 
@@ -149,9 +198,10 @@ class _Ticker:
 
         - N/A
 
-        Returns:
+        # TODO: Add period parameter to financial statements
+
         """
-        return self._get_data("balance-sheet-statement")
+        return self._get_data("balance-sheet-statement", period=period)
 
     def balance_sheet_growth(self) -> List[Dict]:
         """
@@ -176,11 +226,10 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("balance-sheet-statement-growth")
 
-    def cash_flow(self) -> List[Dict]:
+    def cash_flow(self, period: str = None) -> List[Dict]:
         """
         Get cash flow statements
 
@@ -203,9 +252,34 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
-        return self._get_data("cash-flow-statement")
+        return self._get_data("cash-flow-statement", period=period)
+
+    def cash_flow_ttm(self) -> List[Dict]:
+        """
+        Get cash flow statement trailing twelve months
+
+        Exchanges supported
+
+        - NASDAQ
+        - NYSE (New York Stock Exchange)
+        - XETRA (German Electronic Exchange)
+        - ASX (Australian Stock Exchange)
+        - TSX (Toronto Stock Exchange)
+        - ENX (EuroNext)
+        - NSE (National Stock Exchange of India)
+        - LSE (London Stock Exchange)
+        - MOEX (Moscow Stock Exchange)
+        - HKEX (Hong Kong Stock Exchange)
+        - SIX (Swiss Stock Exchange)
+        - OSE (Oslo Stock Exchange)
+
+        Not Supported.
+
+        - N/A
+
+        """
+        return self._get_data("cash-flow-statement-ttm")
 
     def cash_flow_growth(self) -> List[Dict]:
         """
@@ -230,7 +304,6 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("cash-flow-statement-growth")
 
@@ -256,7 +329,6 @@ class _Ticker:
 
         - OSE (Oslo Stock Exchange)
 
-        Returns:
         """
         return self._get_data("ratios")
 
@@ -282,7 +354,6 @@ class _Ticker:
 
         - OSE (Oslo Stock Exchange)
 
-        Returns:
         """
         return self._get_data("enterprise-values")
 
@@ -309,7 +380,6 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("key-metrics")
 
@@ -336,7 +406,6 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("analyst-estimates")
 
@@ -362,7 +431,6 @@ class _Ticker:
 
         - OSE (Oslo Stock Exchange)
 
-        Returns:
         """
         return self._get_data("shares-float")
 
@@ -388,7 +456,6 @@ class _Ticker:
 
         - OSE (Oslo Stock Exchange)
 
-        Returns:
         """
         return self._get_data("quote")
 
@@ -414,7 +481,6 @@ class _Ticker:
 
         - OSE (Oslo Stock Exchange)
 
-        Returns:
         """
         return self._get_data("profile")
 
@@ -428,21 +494,21 @@ class _Ticker:
         - NYSE (New York Stock Exchange)
         - XETRA (German Electronic Exchange)
         - ASX (Australian Stock Exchange)
+        - NSE (National Stock Exchange of India)
+        - LSE (London Stock Exchange)
+        - TSX (Toronto Stock Exchange)
 
         Not supported.
 
-        - TSX (Toronto Stock Exchange)
         - ENX (EuroNext)
-        - NSE (National Stock Exchange of India)
-        - LSE (London Stock Exchange)
         - MOEX (Moscow Stock Exchange)
         - HKEX (Hong Kong Stock Exchange)
         - SIX (Swiss Stock Exchange)
 
-        Returns:
+        Args:
+            months: Get transactions by last n months
         """
 
-        # TODO: Add months parameter.
         return self._get_data("insider-transactions", months=months)
 
     def insider_transactions_summarized(self, months: int = None) -> List[Dict]:
@@ -455,20 +521,21 @@ class _Ticker:
         - NYSE (New York Stock Exchange)
         - XETRA (German Electronic Exchange)
         - ASX (Australian Stock Exchange)
+        - NSE (National Stock Exchange of India)
+        - LSE (London Stock Exchange)
+        - TSX (Toronto Stock Exchange)
 
         Not supported.
 
-        - TSX (Toronto Stock Exchange)
         - ENX (EuroNext)
-        - NSE (National Stock Exchange of India)
-        - LSE (London Stock Exchange)
         - MOEX (Moscow Stock Exchange)
         - HKEX (Hong Kong Stock Exchange)
         - SIX (Swiss Stock Exchange)
 
-        Returns:
+        Args:
+            months: Get transactions by last n months
+
         """
-        # TODO: Add months parameter.
         return self._get_data("insider-transactions-summarized", months=months)
 
     def share_ownership(self) -> List[Dict]:
@@ -494,7 +561,6 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("share-ownership")
 
@@ -521,10 +587,34 @@ class _Ticker:
 
         - N/A
 
-        Returns:
         """
         return self._get_data("key-executives")
 
+    def historical_price(self) -> List[Dict]:
+        """
+        Get historical price data
+
+        Exchanges supported
+
+        - NASDAQ
+        - NYSE (New York Stock Exchange)
+        - XETRA (German Electronic Exchange)
+        - ASX (Australian Stock Exchange)
+        - TSX (Toronto Stock Exchange)
+        - ENX (EuroNext)
+        - NSE (National Stock Exchange of India)
+        - LSE (London Stock Exchange)
+        - MOEX (Moscow Stock Exchange)
+        - HKEX (Hong Kong Stock Exchange)
+        - SIX (Swiss Stock Exchange)
+
+        Not Supported.
+
+        - OSE (Oslo Stock Exchange)
+
+        """
+        return self._get_data("daily-price")
+
 
 class Quantel(object):
 
@@ -533,7 +623,7 @@ class Quantel(object):
         Authenticate with the Quantel Finance API
 
         Args:
-            api_key: Quantel Finance API Key
+            api_key: Quantel Finance API Key - http://links.quantel.io/getstarted
             validate: Validate API Key
 
         Example:
@@ -549,20 +639,23 @@ class Quantel(object):
             self._validate_api()
 
     def _validate_api(self) -> bool:
+        """
+        401 is an Unauthorized status code.
+        403 is an Access Forbidden code.
+
+        Both indicate that the API key was not accepted by RapidAPI
+
+        503 indicates that the API gateway could not be reached.
+        """
 
         headers = {
             "x-rapidapi-key": self.api_key,
-            "user-agent": "Quantel Python Library v0.1"
+            "user-agent": "Quantel Python Library"
         }
         res = requests.get(self.host, headers=headers)
 
         if res.status_code in (401, 403):
-            """
-            401 is an Unauthorized status code. 
-            403 is an Access Forbidden code.
-            
-            Both indicate that the API key was not accepted by RapidAPI
-            """
+
             raise InvalidAPIKey(
                 "Your API Key is invalid. You may have entered your API Key incorrectly, or have not subscribed to the API.\n"
                 "https://quantel.io/faq#invalid_api_key")
@@ -573,11 +666,7 @@ class Quantel(object):
                 "Unable to connect to the API server. If the error persists, please reach out to the team at contact@quantel.io"
             )
 
-        elif res.status_code == 404:
-            """
-            404 not found indicates that RapidAPI accepted the API key in this case.
-            """
-            return True
+        return True
 
     def ticker(self, symbols: Union[list, str], asynchronous: bool = False) -> _Ticker:
         """
